/*
 *  Jajuk
 *  Copyright (C) 2005 Bertrand Florat
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  $Revision: 1.11 $
 */

package org.jajuk.ui;

import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.table.DefaultTableModel;

import org.jajuk.base.IPropertyable;
import org.jajuk.util.ITechnicalStrings;
import org.jajuk.util.Util;

/**
 *  Jajuk table model, adds identifier to model
 *
 * @author     Administrateur
 * @created    22 juin 2005
 */
public abstract class JajukTableModel extends DefaultTableModel  implements ITechnicalStrings{

    /**Column identifiers*/
    volatile public Vector vId = new Vector(10);
    
    /**Rows number*/
    public int iRowNum;
    
    /**Values table**/
    public Object[][] oValues;
    
    //Play icon in cach
    public static final ImageIcon PLAY_ICON = Util.getIcon(ICON_PLAY2);
    
    //Unmount Play icon in cach
    public static final ImageIcon UNMOUNT_PLAY_ICON = Util.getIcon(ICON_UNKNOWN);
    
    /** Objects*/
    public IPropertyable[] oItems;
    
    /**Number of standard columns*/
    public int iNumberStandardCols;
    
    /**Cell editable flag*/
    public boolean[][] bCellEditable;
    
    /**Column names*/
    public Vector vColNames  = new Vector(10);
    
    /**Last value used for undo*/
    public Object oLast = null;
    
    /**Editable flag*/
    boolean bEditable = false;
    
    /**
     * 
     * @param iNumberStandardCols Number of columns of this model (without custom properties)
     */
    public JajukTableModel(int iNumberStandardCols){
        this.iNumberStandardCols = iNumberStandardCols;
    }
    
    /**
     * 
     * Default constructor
     */
    public JajukTableModel(){
        this.iNumberStandardCols = 0;
    }
    
     /**
     * @param sColName
     * @return Column identifier for a given column title
     */
    public String getIdentifier(String sColName){
        return (String)vId.get(vColNames.indexOf(sColName));
    }
    
    /** 
     * Return item at given position
     * @param iRow
     * @return
     */
    public IPropertyable getItemAt(int iRow){
        return oItems[iRow];
    }
    
    /** 
     * Set item at given position
     * @param iRow
     * @param IPropertyabe item to set
     */
    public void setItemAt(int iRow,IPropertyable item){
        oItems[iRow]=item;
    }
        
    public Object getValueAt(int rowIndex, int columnIndex) {
        return oValues[rowIndex][columnIndex];
    }
    
    public  void setValueAt(Object oValue, int rowIndex, int columnIndex) {
        oLast = oValues[rowIndex][columnIndex];
        oValues[rowIndex][columnIndex] = oValue;
        fireTableCellUpdated(rowIndex,columnIndex);
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount() {
        return vColNames.size();
    }
    
    /**
     * Undo last change
     *
     */
    public void undo(int rowIndex, int columnIndex) {
        if (oLast != null){
            oValues[rowIndex][columnIndex] = oLast;
        }
    }
    
    
    public String getColumnName(int column){
        return (String)vColNames.get(column);
    }
    
    public String getIdentifier(int column){
        return (String)vId.get(column);
    }
    
    public int getRowCount() {
        return iRowNum;
    }
    
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return bEditable && bCellEditable[rowIndex][columnIndex];
    }
    
    public Class getColumnClass(int columnIndex) {
        Object o = getValueAt(0,columnIndex);
        if ( o != null){
            return o.getClass();
        }
        else{
            return null;
        }
    }
    
    /**
     * Fill model with data using an optionnal filter property and pattern
    * @param sProperty Property (column) to filter
    * @param sPattern pattern*/
    public abstract void populateModel(String sProperty,String sPattern); 
    
    
     /**
     * Fill model with data
   */
    public void populateModel(){
        populateModel(null,null);
    }
    
    /** 
     * Set this model editable state
     *@param b whether model is editable or not
     **/
    public void setEditable(boolean b){
        this.bEditable = b;
    }
    
                
}
