/*
 *  Jajuk
 *  Copyright (C) 2003 Bertrand Florat
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  $Revision: 1.26 $
 */

package org.jajuk.ui.views;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;

import org.jajuk.base.Event;
import org.jajuk.base.FIFO;
import org.jajuk.base.ObservationManager;
import org.jajuk.base.Observer;
import org.jajuk.base.PlaylistFile;
import org.jajuk.base.PlaylistFileManager;
import org.jajuk.base.PlaylistManager;
import org.jajuk.i18n.Messages;
import org.jajuk.ui.PlaylistFileItem;
import org.jajuk.ui.PropertiesWizard;
import org.jajuk.util.Util;
import org.jajuk.util.error.JajukException;
import org.jajuk.util.log.Log;

import ext.SwingWorker;

/**
 * Shows playlist files
 * <p>Physical perspective
 *  * <p>Singleton
 * @author     Bertrand Florat
 * @created   29 dec. 2003
 */
abstract public class AbstractPlaylistRepositoryView extends ViewAdapter implements Observer,ActionListener{
	
	
	/**Selected playlist file item */
	PlaylistFileItem plfiSelected;
	
	/**Queue playlist*/
	PlaylistFileItem plfiQueue;
	
	/**New playlist*/
	PlaylistFileItem plfiNew;
	
	/**Queue playlist*/
	PlaylistFileItem plfiBookmarks;
	
	/**Bestof playlist*/
	PlaylistFileItem plfiBestof;
    
    /**Novelties playlist*/
    PlaylistFileItem plfiNovelties;
    
	/**List of playlistfile item*/
	ArrayList<PlaylistFileItem> alPlaylistFileItems = new ArrayList(10);
	
	/**Concorency flag*/
    boolean bReleased = true;
    
	JPanel jpRoot;
	JPopupMenu jpmenu;
	JMenuItem jmiPlay;
	JMenuItem jmiSaveAs;
	JMenuItem jmiDelete;
	JMenuItem jmiProperties;
	
	MouseAdapter ma;
	
	
	/* (non-Javadoc)
	 * @see org.jajuk.ui.IView#display()
	 */
	public void populate(){
		setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
		
		//root pane
		jpRoot = new JPanel();
		jpRoot.setLayout(new BoxLayout(jpRoot,BoxLayout.Y_AXIS));
		
		//Popup menus
		jpmenu =  new JPopupMenu();
		
		jmiPlay = new JMenuItem(Messages.getString("PhysicalPlaylistRepositoryView.0"));  //$NON-NLS-1$
		jmiPlay.addActionListener(this);
		jpmenu.add(jmiPlay);
		
		jmiDelete = new JMenuItem(Messages.getString("PhysicalPlaylistRepositoryView.3"));  //$NON-NLS-1$
		jmiDelete.addActionListener(this);
		jpmenu.add(jmiDelete);
		
		jmiSaveAs = new JMenuItem(Messages.getString("PhysicalPlaylistRepositoryView.2"));  //$NON-NLS-1$
		jmiSaveAs.addActionListener(this);
		jpmenu.add(jmiSaveAs);
		
		jmiProperties = new JMenuItem(Messages.getString("PhysicalPlaylistRepositoryView.4"));  //$NON-NLS-1$
		jmiProperties.addActionListener(this);
		jpmenu.add(jmiProperties);
		
		//mouse adapter
		ma = new MouseAdapter() {
			public void mouseClicked(final MouseEvent e) {
				PlaylistFileItem plfi = (PlaylistFileItem)e.getComponent();
				if (plfi == plfiSelected){
					if (e.getButton() == MouseEvent.BUTTON3){  //right button
						showMenu(plfi,e);
						return;
					}
					else { //left button again launch it 
					    try{
					        play(plfi);
                        }
					    catch(JajukException je){
                            Log.error(je.getCode(),plfiSelected.getName(),null); //$NON-NLS-1$
					        Messages.showErrorMessage(je.getCode(),plfiSelected.getName()); //$NON-NLS-1$
					        selectQueue();
                            return;
					    }
                   }
				}
				else{ //we selected a new playlist file
				    try{
				        plfi.getPlaylistFile().getFiles();
				    }
				    catch(JajukException je){
				        Log.error(je.getCode(),plfi.getName(),null); //$NON-NLS-1$
				        Messages.showErrorMessage(je.getCode(),plfi.getName()); //$NON-NLS-1$
				        selectQueue();
				        return;
				    }
				    Util.waiting();
				    selectPlaylistFileItem(plfi);
					if (e.getButton() == MouseEvent.BUTTON3){  //right button
						showMenu(plfi,e);
					}
				}		
			}
		};
		//refresh
		populatePlaylists();
		jpRoot.add(Box.createVerticalStrut(500));  //make sure playlists items are packed to the top
		JScrollPane jsp = new JScrollPane(jpRoot);
        jsp.getVerticalScrollBar().setUnitIncrement(60);
        add(jsp);
		//Register on the list for subject we are interrested in
		ObservationManager.register(EVENT_DEVICE_MOUNT,this);
		ObservationManager.register(EVENT_DEVICE_UNMOUNT,this);
		ObservationManager.register(EVENT_DEVICE_REFRESH,this);
		//set queue playlist as default in playlist editor
		selectPlaylistFileItem(plfiQueue);	
	}
	
	
	private void selectQueue(){
	    selectPlaylistFileItem(plfiQueue);
	}
    
    
	/**
	 * Display the playlist menu
	 * @param plfi
	 * @param e
	 */
	private void showMenu(PlaylistFileItem plfi,MouseEvent e){
		if (plfi.getType() != PlaylistFileItem.PLAYLIST_TYPE_NORMAL){
			//cannot delete special playlists
			jmiDelete.setEnabled(false); 
			//Save as is only for special playlists
			jmiSaveAs.setEnabled(true);
            jmiProperties.setEnabled(false);
		}
		else{
			jmiDelete.setEnabled(true);
			jmiSaveAs.setEnabled(false);
            jmiProperties.setEnabled(true);
		}
		//cannot play the queue playlist : nonsense
		if ( plfiSelected.getType() == PlaylistFileItem.PLAYLIST_TYPE_QUEUE){
			jmiPlay.setEnabled(false);  
		}
		else{
			jmiPlay.setEnabled(true);
		}
		jpmenu.show(e.getComponent(),e.getX(),e.getY());
		
	}
	
	/**
	 * Set current playlist file item
	 * @param plfi
	 */
	synchronized void selectPlaylistFileItem(PlaylistFileItem plfi){
		//remove item border
		if (plfiSelected!=null){
			plfiSelected.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		}
		plfi.setBorder(BorderFactory.createLineBorder(Color.BLACK,5));
		//set new item
		this.plfiSelected = plfi;
        FIFO.getInstance().setPlaylist(plfi.getPlaylistFile());
        Properties properties = new Properties();
        properties.put(DETAIL_ORIGIN,AbstractPlaylistRepositoryView.this);
        properties.put(DETAIL_SELECTION,plfi);
        ObservationManager.notify(new Event(EVENT_PLAYLIST_CHANGED,properties));
  }
	
	
	
	/* (non-Javadoc)
	 * @see org.jajuk.ui.IView#getDesc()
	 */
	public String getDesc() {
		return "PhysicalPlaylistRepositoryView.6";	 //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.jajuk.ui.IView#getViewName()
	 */
	public String getViewName() {
		return "org.jajuk.ui.views.PhysicalPlaylistRepositoryView"; //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.jajuk.ui.Observer#update(java.lang.String)
	 */
	public void update(final Event event) {
		String subject = event.getSubject();
		if ( subject.equals(EVENT_DEVICE_MOUNT) 
                || subject.equals(EVENT_DEVICE_UNMOUNT) 
                || subject.equals(EVENT_DEVICE_REFRESH) ) {
            
            SwingWorker sw = new SwingWorker() {
                public Object  construct(){
                    bReleased = false; //take "MUTEX"
                    if (jpRoot.getComponentCount() > 0){
                        jpRoot.removeAll();
                    }
                    populatePlaylists();
                    //try to keep back previous selected playlist if it exists
                    if (plfiSelected == null || !alPlaylistFileItems.contains(plfiSelected)){
                        plfiSelected = plfiQueue;
                    }
                    return null;
                }
                public void finished() {
					jpRoot.add(Box.createVerticalStrut(500));  //make sure specials playlists are packed to the top
				    if ( plfiSelected.getPlaylistFile().getType() != PlaylistFileItem.PLAYLIST_TYPE_NORMAL ||
                            PlaylistFileManager.getInstance().getItem(plfiSelected.getPlaylistFile().getId()) != null){
				        selectPlaylistFileItem(plfiSelected);    
                    }
                    else{ //means previously selected playlist has changed, select queue
                        selectPlaylistFileItem(plfiQueue);
                    }
                    AbstractPlaylistRepositoryView.this.revalidate();
                    AbstractPlaylistRepositoryView.this.repaint();
                    bReleased = true;
               }
			};
			int i = 0;
            while (!bReleased && i < 100){ //wait until ressource is released
			    //we can't just synchronize because action is done in 2 methods of swing worker
                try {
                    Thread.sleep(100);
                    i++;
                }
                catch (InterruptedException e) {
                    Log.error(e);
                }         
            }
            sw.start();
      }
	}
	
	/**
	 * Create special playlists from collection, this is called by both logicial and physical populate methods
	 */
	 synchronized void populatePlaylists(){
		alPlaylistFileItems.clear();
		//special playlists
		JPanel jpSpecials = new JPanel();
		jpSpecials.setBorder(BorderFactory.createTitledBorder(Messages.getString("PhysicalPlaylistRepositoryView.8"))); //$NON-NLS-1$
		jpSpecials.setLayout(new BoxLayout(jpSpecials,BoxLayout.Y_AXIS));
		//queue
		//note we give an id : this id is only used to match current playlist
		if (plfiQueue == null){
            plfiQueue = new PlaylistFileItem(PlaylistFileItem.PLAYLIST_TYPE_QUEUE,ICON_PLAYLIST_QUEUE,new PlaylistFile(PlaylistFileItem.PLAYLIST_TYPE_QUEUE,"1",null,null),Messages.getString("PhysicalPlaylistRepositoryView.9")); //$NON-NLS-1$ //$NON-NLS-2$
            plfiQueue.setToolTipText(Messages.getString("PhysicalPlaylistRepositoryView.10")); //$NON-NLS-1$
            plfiQueue.addMouseListener(ma);
        }
        else if (plfiSelected!=null && plfiQueue.getPlaylistFile().equals(plfiSelected.getPlaylistFile())){
            plfiSelected = plfiQueue;
        }
        jpSpecials.add(plfiQueue);
		alPlaylistFileItems.add(plfiQueue);
        
		//new
		if (plfiNew == null){
		    plfiNew = new PlaylistFileItem(PlaylistFileItem.PLAYLIST_TYPE_NEW,ICON_PLAYLIST_NEW,new PlaylistFile(PlaylistFileItem.PLAYLIST_TYPE_NEW,"2",null,null),Messages.getString("PhysicalPlaylistRepositoryView.11")); //$NON-NLS-1$ //$NON-NLS-2$
		    plfiNew.setToolTipText(Messages.getString("PhysicalPlaylistRepositoryView.12")); //$NON-NLS-1$
		    plfiNew.addMouseListener(ma);
        }
		else if (plfiSelected!=null && plfiNew.getPlaylistFile().equals(plfiSelected.getPlaylistFile())){
            plfiSelected = plfiNew;
        }
        jpSpecials.add(plfiNew);
		alPlaylistFileItems.add(plfiNew);
		//bookmark
		if (plfiBookmarks == null){
		    plfiBookmarks = new PlaylistFileItem(PlaylistFileItem.PLAYLIST_TYPE_BOOKMARK,ICON_PLAYLIST_BOOKMARK,new PlaylistFile(PlaylistFileItem.PLAYLIST_TYPE_BOOKMARK,"3",null,null),Messages.getString("PhysicalPlaylistRepositoryView.13")); //$NON-NLS-1$ //$NON-NLS-2$
		    plfiBookmarks.setToolTipText(Messages.getString("PhysicalPlaylistRepositoryView.14")); //$NON-NLS-1$
		    plfiBookmarks.addMouseListener(ma);
		}
        else if (plfiSelected!=null && plfiBookmarks.getPlaylistFile().equals(plfiSelected.getPlaylistFile())){
            plfiSelected = plfiBookmarks;
        }
        jpSpecials.add(plfiBookmarks);
		alPlaylistFileItems.add(plfiBookmarks);
        //Best of
		if (plfiBestof == null){
            plfiBestof = new PlaylistFileItem(PlaylistFileItem.PLAYLIST_TYPE_BESTOF,ICON_PLAYLIST_BESTOF,new PlaylistFile(PlaylistFileItem.PLAYLIST_TYPE_BESTOF,"4",null,null),Messages.getString("PhysicalPlaylistRepositoryView.15")); //$NON-NLS-1$ //$NON-NLS-2$
    		plfiBestof.setToolTipText(Messages.getString("PhysicalPlaylistRepositoryView.16")); //$NON-NLS-1$
    		plfiBestof.addMouseListener(ma);
        }
        else if (plfiSelected!=null && plfiBestof.getPlaylistFile().equals(plfiSelected.getPlaylistFile())){
            plfiSelected = plfiBestof;
        }
        jpSpecials.add(plfiBestof);
        alPlaylistFileItems.add(plfiBestof);
        if (plfiSelected!=null && plfiBestof.getPlaylistFile().equals(plfiSelected.getPlaylistFile())){
            plfiSelected = plfiBestof;
        }
        //Novelties
        if (plfiNovelties == null){
            plfiNovelties = new PlaylistFileItem(PlaylistFileItem.PLAYLIST_TYPE_NOVELTIES,ICON_PLAYLIST_NOVELTIES,new PlaylistFile(PlaylistFileItem.PLAYLIST_TYPE_NOVELTIES,"1",null,null),Messages.getString("PhysicalPlaylistRepositoryView.17")); //$NON-NLS-1$ //$NON-NLS-2$
            plfiNovelties.setToolTipText(Messages.getString("PhysicalPlaylistRepositoryView.18")); //$NON-NLS-1$
            plfiNovelties.addMouseListener(ma);
        }
        else if (plfiSelected!=null && plfiNovelties.getPlaylistFile().equals(plfiSelected.getPlaylistFile())){
            plfiSelected = plfiNovelties;
        }
        jpSpecials.add(plfiNovelties);
		alPlaylistFileItems.add(plfiNovelties);
		
		jpRoot.add(jpSpecials);
	}
    
    abstract public void removeItem (PlaylistFileItem plfiSelected);
    
    abstract public void play(PlaylistFileItem plfi) throws JajukException;
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(final ActionEvent ae) {
		new Thread(){
		    public void run(){
		        //Property action is available even for unmounted items
                if(ae.getSource() == jmiProperties){
		            ArrayList alItems = new ArrayList(1);
		            if ( AbstractPlaylistRepositoryView.this instanceof PhysicalPlaylistRepositoryView){
		                alItems.add(plfiSelected.getPlaylistFile());  
		            }
		            else{
		                alItems.add(PlaylistManager.getInstance().getPlayList(plfiSelected.getPlaylistFile()));;
		            }
		            new PropertiesWizard(alItems);
		        }
		        //Others actions: not available for unmounted items
                else{
                    if ( ae.getSource() == jmiDelete){
		                removeItem(plfiSelected);
                        ObservationManager.notify(new Event(EVENT_DEVICE_REFRESH));
		            }
		            else if(ae.getSource() == jmiPlay){
		                try{
		                    play(plfiSelected);
		                }
		                catch(JajukException je){
		                    Log.error(je.getCode(),"{{"+plfiSelected.getName()+"}}",null); //$NON-NLS-1$ //$NON-NLS-2$
		                    Messages.showErrorMessage(je.getCode(),plfiSelected.getName()); //$NON-NLS-1$
                            selectQueue();
                            return;
		                }
		            }
		            else if(ae.getSource() == jmiProperties){
		                ArrayList alItems = new ArrayList(1);
		                if ( AbstractPlaylistRepositoryView.this instanceof PhysicalPlaylistRepositoryView){
		                    alItems.add(plfiSelected.getPlaylistFile());  
		                }
		                else{
		                    alItems.add(PlaylistManager.getInstance().getPlayList(plfiSelected.getPlaylistFile()));;
		                }
		                new PropertiesWizard(alItems);
		            }
		            else if(ae.getSource() == jmiSaveAs){ //save as
		                try{
		                    plfiSelected.getPlaylistFile().saveAs();
                            ObservationManager.notify(new Event(EVENT_DEVICE_REFRESH)); //notify playlist repository to refresh
                        }
                        
                        catch(JajukException je){
                            Log.error(je);
                            Messages.showErrorMessage(je.getCode());
                        }
                        catch(Exception e){
                            Log.error(e);
                        }
                        ObservationManager.notify(new Event(EVENT_PLAYLIST_REFRESH));
		            }
		        }
		    }
		}.start();
	}
	
	
	/**
	 * @return Returns the alPlaylistFileItems.
	 */
	public ArrayList getPlaylistFileItems() {
		return alPlaylistFileItems;
	}
	
	/**
	 * @return Returns the plfiQueue.
	 */
	public PlaylistFileItem getQueue() {
		return plfiQueue;
	}
	
}
